/***************************************************************************

  2610intf.c

  The YM2610 emulator supports up to 2 chips.
  Each chip has the following connections:
  - Status Read / Control Write A
  - Port Read / Data Write A
  - Control Write B
  - Data Write B

***************************************************************************/


#include <stdio.h>
#include "../neocd.h"
#include "ay8910.h"
#include "fm.h"
#include "2610intf.h"
#include "timer.h"

/* use FM.C with stream system */

static int stream;

static timer_struct *Timer[2];

/*------------------------- TM2610 -------------------------------*/
/* IRQ Handler */
static void IRQHandler(int irq)
{
    //printf("IRQ!!!\n");
    neogeo_sound_irq(irq);
}

/* Timer overflow callback from timer.c */
void timer_callback_2610(int param)
{
    Timer[param] = 0;
    YM2610TimerOver(param);
}

/* TimerHandler from fm.c */
static void TimerHandler(int c, int count, double stepTime)
{
    //printf("TimerHandler %d %d %f\n",c,count,stepTime);
    if (count == 0) {		/* Reset FM Timer */
	if (Timer[c]) {
	    del_timer(Timer[c]);
	    Timer[c] = 0;
	}
    } else {			/* Start FM Timer */
	double timeSec = (double) count * stepTime;

	if (Timer[c] == 0) {
	    Timer[c] =	(timer_struct *) insert_timer(timeSec, c,timer_callback_2610);
	}
    }
}
void FMTimerInit(void)
{
    Timer[0] = Timer[1] = 0;
    free_all_timer();
}

int YM2610_sh_start(void)
{
    int j;
    char buf[YM2610_NUMBUF][40];
    void *pcmbufa, *pcmbufb;
    int pcmsizea, pcmsizeb;

    AY8910_sh_start();


    /* Timer Handler set */
    FMTimerInit();
    for (j = 0; j < YM2610_NUMBUF; j++) {
	buf[j][0] = 0;
    }
    stream = stream_init_multi(2, 0, YM2610UpdateOne);
    pcmbufa = (void *) neogeo_pcm_memory;
    pcmsizea = 0x100000; //1048576;
    pcmbufb = NULL; /*(void *) memory.sound2;*/
    pcmsizeb = 0; /*memory.sound2_size;*/
	

    //}

   
	return (YM2610Init(8000000, SAMPLE_RATE, pcmbufa, pcmsizea, pcmbufb, pcmsizeb,  TimerHandler, IRQHandler) );

    /* error */
	printf("YM2610Init failed\n");
    return 1;
}
/*	//unrolled into the port handlers
u8 YM2610Read(int a)
{
    YM2610 *F2610 = &(FM2610);
    int addr = F2610->OPN.ST.address;
    u8 ret = 0;

    switch (a & 3) {
    case 0:			// status 0 : YM2203 compatible 
		ret = F2610->OPN.ST.status & 0x83;
	break;
	case 1:			// data 0 
	if (addr < 16)
	    ret = SSGRead();
	if (addr == 0xff)
	    ret = 0x01;
	break;
    case 2:			// status 1 : + ADPCM status 
	// ADPCM STATUS (arrived End Address) 
	// B,--,A5,A4,A3,A2,A1,A0 
	// B     = ADPCM-B(DELTA-T) arrived end address 
	// A0-A5 = ADPCM-A          arrived end address 
	ret = F2610->adpcm_arrivedEndAddress | F2610->deltaT.arrivedFlag;
	break;
    case 3:	
		ret = 0;	
	break;	
    }
    return ret;
}
*/
/************************************************/
/* Sound Hardware Stop				*/
/************************************************/
//void YM2610_sh_stop(void){}
/************************************************/
/* Status Read for YM2610 - Chip 0		*/
/************************************************/
u32 YM2610_status_port_0_A_r(u32 offset)
{
    //return YM2610Read(0);
	YM2610 *F2610 = &(FM2610);
	return F2610->OPN.ST.status & 0x83;
}

u32 YM2610_status_port_0_B_r(u32 offset)
{
    //return YM2610Read(2);
	YM2610 *F2610 = &(FM2610);
	return F2610->adpcm_arrivedEndAddress | F2610->deltaT.arrivedFlag;
}

/************************************************/
/* Port Read for YM2610 - Chip 0		*/
/************************************************/
u32 YM2610_read_port_0_r(u32 offset)
{
    //return YM2610Read(1);
	YM2610 *F2610 = &(FM2610);
    int addr = F2610->OPN.ST.address;
	if(addr < 16)
		return SSGRead();
	if(addr == 0xff)
		return 0x01;
}


/************************************************/
/* Control Write for YM2610 - Chip 0		*/
/* Consists of 2 addresses			*/
/************************************************/
void YM2610_control_port_0_A_w(u32 offset, u32 data)
{
    YM2610Write(0, data);
}

void YM2610_control_port_0_B_w(u32 offset, u32 data)
{
    YM2610Write(2, data);
}

/************************************************/
/* Data Write for YM2610 - Chip 0		*/
/* Consists of 2 addresses			*/
/************************************************/
void YM2610_data_port_0_A_w(u32 offset, u32 data)
{
    YM2610Write(1, data);
}

void YM2610_data_port_0_B_w(u32 offset, u32 data)
{
    YM2610Write(3, data);
}
