/***************************************************************************

  streams.c

  Handle general purpose audio streams

***************************************************************************/
#include "../neocd.h"
#include "sound.h"
#define MIXER_MAX_CHANNELS 5

#define CPU_FPS 60

extern int frame;

uint16 left_buffer[BUFFER_LEN/2]__attribute__((aligned(32)));
uint16 right_buffer[BUFFER_LEN/2]__attribute__((aligned(32)));


int SamplePan[] = { 0, 255, 128, 0, 255 };

void MixAudio (u8 *dst, const u8 *src, u32 len);
static int stream_joined_channels[MIXER_MAX_CHANNELS]__attribute__((aligned(32)));

static int stream_vol[MIXER_MAX_CHANNELS]__attribute__((aligned(32)));

struct {
    Sint16 *buffer;
    void (*callback) (int param, Sint16 ** buffer, int length);
} stream[MIXER_MAX_CHANNELS]__attribute__((aligned(32)));

void mixer_set_volume(int channel,int volume) {
    stream_vol[channel]=volume;
}

int streams_sh_start(void)
{
    int i;

    for (i = 0; i < MIXER_MAX_CHANNELS; i++) {
	stream_joined_channels[i] = 1;
	stream[i].buffer = 0;
    }
    return 0;
}


void streams_sh_stop(void)
{
    int i;

    for (i = 0; i < MIXER_MAX_CHANNELS; i++) {
	free(stream[i].buffer);
	stream[i].buffer = 0;
    }
}

void streamupdate(int len)
{
    /*static int current_pos;*/
    int channel, i;
    Uint16 *bl, *br;
    Uint16 *pl;

    /* update all the output buffers */
   sq_clr(left_buffer,  BUFFER_LEN/2);
   sq_clr(right_buffer,  BUFFER_LEN/2);

    for (channel = 0; channel < MIXER_MAX_CHANNELS;
	 channel += stream_joined_channels[channel]) {
	if (stream[channel].buffer) {
	    int buflen;

	    buflen = len >> 2;

	    if (stream_joined_channels[channel] > 1) {
		Sint16 *buf[MIXER_MAX_CHANNELS]__attribute__((aligned(32)));

		if (buflen > 0) {
		    for (i = 0; i < stream_joined_channels[channel]; i++)
			buf[i] = stream[channel + i].buffer;


		    (*stream[channel].callback) (0,
						 buf, buflen);
		}

	    }
	}
    }

    bl = left_buffer;
    br = right_buffer;

    for (channel = 0; channel < MIXER_MAX_CHANNELS; channel += stream_joined_channels[channel]) {

		if (stream[channel].buffer) {

			for (i = 0; i < stream_joined_channels[channel]; i++) {

			if (SamplePan[channel + i] <= 128)
			    MixAudio((Uint8 *) bl,(Uint8 *) stream[channel + i].buffer,	 len >> 1);
			if (SamplePan[channel + i] >= 128)
			    MixAudio((Uint8 *) br,(Uint8 *) stream[channel + i].buffer,	 len >> 1);

			}
		}
    }
}

void streamupdatemono(void)
{
    /*static int current_pos;*/
    int channel, i;

    /* update all the output buffers */
   sq_clr(left_buffer,  BUFFER_LEN/2);
   
    for (channel = 0; channel < MIXER_MAX_CHANNELS;	 channel += stream_joined_channels[channel]) {

		Sint16 *buf[MIXER_MAX_CHANNELS];

		for (i = 0; i < stream_joined_channels[channel]; i++)
			buf[i] = stream[channel + i].buffer;


		(*stream[channel].callback) (0,	 buf, SAMPLES);

    }

    for (channel = 0; channel < MIXER_MAX_CHANNELS; channel++) 
		MixAudio((Uint8 *) left_buffer,(Uint8 *) stream[channel].buffer, BUFFER_LEN/2);
    
}

int stream_init_multi(int channels, int param,
		      void (*callback) (int param, Sint16 ** buffer,
					int length))
{
    static int channel, i;

    stream_joined_channels[channel] = channels;

    for (i = 0; i < channels; i++) {

	if ((stream[channel + i].buffer =
	     memalign(32,sizeof(Sint16) * BUFFER_LEN)) == 0)
	    return -1;
    }

    stream[channel].callback = callback;
    channel += channels;
    return channel - channels;
}

